function snap = AdjointActDisk(bU,bV,C1,Re,Cd,dt,nx,ny,hx,hy,X,Y,Nt,m,E,locTurb1,locTurb2,locTurbY)

%-----------------------------------------------------------------------
% initial conditions

U = zeros(nx,ny); 
V = zeros(nx,ny);

% initial perturbation to break symmetry
%U = U + 0.1*exp(-25*((X-12).^2+(Y-2).^2));

U = reshape(U,[],1);
V = reshape(V,[],1);

U = full(C1(1,1:nx*ny)');
V = full(C1(1,nx*ny+1:2*nx*ny)');
snap = zeros(2*nx*ny,60);

%-----------------------------------------------------------------------
% setup 

% first derivatives
Dx = kron(speye(ny),D1z(nx,hx));
Dy = kron(D1p(ny,hy),speye(nx));

% Laplacian
L = kron(speye(ny),D2z(nx,hx)) + kron(D2p(ny,hy),spdiags([0;ones(nx-2,1);0],0,nx,nx));

% Cholesky requires positive definite so use -L
Lp = -L;
perp = symamd(Lp); 
Rp = chol(Lp(perp,perp)); 
Rpt = Rp';


%-----------------------------------------------------------------------
% time loop 

for step = 1:Nt

    % linear drag force (not realistic) 
    u_ib = E*U;
    v_ib = E*V;   
    fx = -u_ib;
    fy = -v_ib;
    
    % predict velocity (forward Euler)
    Ut = U + dt * (1/Re*(L*U) + Dx*(bU.*U) - U.*(Dx*bU) + Dy*(bV.*U) - V.*(Dx*bV) + Cd*E'*fx);
    Vt = V + dt * (1/Re*(L*V) + Dx*(bU.*V) - U.*(Dy*bU) + Dy*(bV.*V) - V.*(Dy*bV) + Cd*E'*fy);
    
    % boundary conditions (Dirichlet inflow, convective outflow)
    Ut = reshape(Ut,nx,ny);
    Vt = reshape(Vt,nx,ny);
    Ut(end,:) = 0;
    Vt(end,:) = 0;
    Ut(1,:) = U(1,:) + dt/hx*(U(2,:)-U(1,:));
    Vt(1,:) = V(1,:) + dt/hx*(V(2,:)-V(1,:));
    Ut = reshape(Ut,[],1);
    Vt = reshape(Vt,[],1);
   
    % Poisson eqn for pressure
    rhs = (Dx*Ut + Dy*Vt)/dt;        
    P(perp,1) = -Rp\(Rpt\rhs(perp));
    
    % correct velocity
    U = Ut - dt*Dx*P;
    V = Vt - dt*Dy*P;
    
    
    resid(step) = norm(U);
    
    % visualization
    if (mod(step,m) == 0)
        pcolor(X,Y,reshape(U,nx,ny)); shading interp; colorbar;
        daspect([1 1 1]);
        hold on;
        plot([locTurb1 locTurb1],[locTurbY-0.5 locTurbY+0.5],'k','LineWidth',2);
        plot([locTurb2 locTurb2],[locTurbY-0.5 locTurbY+0.5],'k','LineWidth',2);
%         plot([12 12],[2 3],'k','LineWidth',2);
%         plot([16 16],[2 3],'k','LineWidth',2);
%         plot([8 8],[7 8],'k','LineWidth',2);
%         plot([12 12],[7 8],'k','LineWidth',2);
%         plot([16 16],[7 8],'k','LineWidth',2);
%         plot([8 8],[11 12],'k','LineWidth',2);
%         plot([12 12],[11 12],'k','LineWidth',2);
%         plot([16 16],[11 12],'k','LineWidth',2);
        hold off;
        drawnow;
        fprintf('%d %e\n', step, resid(step));
        snap(:,step/m) = [U; V];
    end;
    
    
    
end;
    
figure;
semilogy(resid);

end