function F = rolloff(wbar, varargin)
% ROLLOFF(wbar, n) n-th order low pass filter with gain = 1 for w < wbar
%                 using the Butterworth pattern.
%
%                                      1
% parametrized transfer function    ----------
%                                     B(n)
%
%   wbar: (double) value of cutoff frequency
%      n: (double) degree of rolloff
%
% if no n is specified n = 1 is assumed.

if nargin == 1
    F = ss(tf(wbar, [1 wbar]));
elseif nargin == 2
    if varargin{1} < 1
        error('n has to be positive')
    end
    if abs(varargin{1}-int8(varargin{1})) > 10^(-12)
        error('n has to be an integer')
    end
    
B = butterworthpattern(wbar,varargin{1});

F = ss(tf(1,B));
end



function B = butterworthpattern(R,n)
% BUTTERWORTHPATTERN(R,n) returns the polynomial with roots matching the
% Butterworth pattern; i.e. all roots are on a circle with radius R in
% the left half plane with angular distance pi/n between them.

 B=1; % initialize polynomial
    for k = 1:n
    zk = R * exp(1j*(2*k+n-1)*pi/(2*n)); % compute nth root location
    B= conv(B, [1 -zk]); % multiply polynomial
    end
    
% remove numerical noise causing asymmetry in root's locations
B=real(B);

% normalize polynomial coefficients to 0th degree term = 1
for i = 1:numel(B)
    B(i) = B(i)/B(end);
end